@@ -18,16 +18,19 @@ class EventSentinel:
             try:
                 self.am_subscribers.remove(listener)
             except ValueError:
-                l.warning(f"Double-unsubscribe of {listener} from {self}")
+                l.warning("Double-unsubscribe of %s from %s", listener, self)
 
     def am_event(self, **kwargs):
         for listener in self.am_subscribers:
-            listener(**kwargs)
+            try:
+                listener(**kwargs)
+            except Exception:  # pylint: disable=broad-except
+                l.exception("Error raised from event of %s", self)
 
 
 class ObjectContainer(EventSentinel):
     def __init__(self, obj, name=None, notes=''):
-        super(ObjectContainer, self).__init__()
+        super().__init__()
         self._am_obj = None
         self.am_obj = obj
         self.am_name = name if name is not None else NameGenerator.random_name()
@@ -62,7 +65,7 @@ class ObjectContainer(EventSentinel):
     def __setattr__(self, key, value):
         if key.startswith('am_') or key.startswith('_am_'):
             return object.__setattr__(self, key, value)
-        setattr(self._am_obj, key, value)
+        return setattr(self._am_obj, key, value)
 
     def __getitem__(self, item):
         return self._am_obj[item]
@@ -83,7 +86,7 @@ class ObjectContainer(EventSentinel):
         return self is other or self._am_obj == other
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self == other
 
     def __repr__(self):
         return '(container: %s)%s' % (self.am_name, repr(self._am_obj))
