@@ -1,6 +1,7 @@
 from typing import Optional
 from collections import OrderedDict
 
+from PySide2.QtCore import Qt
 from PySide2.QtWidgets import QDialog, QLineEdit, QPushButton, QHBoxLayout, QVBoxLayout, QMessageBox
 
 import pycparser.plyparser
@@ -34,6 +35,9 @@ class CTypeEditor(QDialog):
         self._ok_button = None  # type: Optional[QPushButton]
         self._init_widgets(base_text, multiline)
 
+        self.setWindowTitle("Type editor")
+        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
+
         self.result = []
 
     def _init_widgets(self, base_text, multiline):
