@@ -2,14 +2,18 @@ from typing import Sequence, Union, Optional, Tuple, Callable
 import logging
 
 import PySide2
-
 from PySide2.QtWidgets import QApplication, QHBoxLayout, QMainWindow, QVBoxLayout, QFrame, QGraphicsView, \
-                              QGraphicsScene, QGraphicsItem, QGraphicsObject, QGraphicsSimpleTextItem, \
-                              QGraphicsSceneMouseEvent, QLabel
-from PySide2.QtGui import QPainterPath, QPen, QFont, QColor, QWheelEvent
+    QGraphicsScene, QGraphicsItem, QGraphicsObject, QGraphicsSimpleTextItem, \
+    QGraphicsSceneMouseEvent, QLabel, QMenu, QPushButton, QAction, QMessageBox
+from PySide2.QtGui import QPainterPath, QPen, QFont, QColor, QWheelEvent, QCursor
 from PySide2.QtCore import Qt, QRectF, QPointF, QSizeF, Signal, QEvent, QMarginsF, QTimer
 
-from .view import BaseView
+from angr import Block
+from angr.knowledge_plugins.cfg import MemoryData
+from angr.knowledge_plugins.patches import Patch
+from angrmanagement.ui.dialogs.input_prompt import InputPromptDialog
+
+from .view import SynchronizedView
 from ..dialogs.jumpto import JumpTo
 from ...utils import is_printable
 from ...config import Conf
@@ -22,6 +26,7 @@ HexAddress = int
 HexDataBuffer = Union[bytes, bytearray]
 HexDataProvider = Callable[[HexAddress], HexByteValue]
 
+
 class HexHighlightRegion:
     """
     Defines a highlighted region.
@@ -33,14 +38,117 @@ class HexHighlightRegion:
         self.size: int = size
         self.active: bool = False
 
+    def gen_context_menu_actions(self) -> Optional[QMenu]:  # pylint: disable=no-self-use
+        """
+        Get submenu for this highlight region.
+        """
+        return None
+
+
+class PatchHighlightRegion(HexHighlightRegion):
+    """
+    Defines a highlighted region indicating a patch.
+    """
+
+    def __init__(self, patch: Patch, view: 'HexView'):
+        super().__init__(Qt.yellow, patch.addr, len(patch))
+        self.patch: Patch = patch
+        self.view: 'HexView' = view
+
+    def gen_context_menu_actions(self) -> Optional[QMenu]:
+        """
+        Get submenu for this highlight region.
+        """
+        mnu = QMenu(f"Patch 0x{self.patch.addr:x} ({len(self.patch)} bytes)")
+        act = QAction("&Split", mnu)
+        act.triggered.connect(self.split)
+        act.setEnabled(self.can_split())
+        mnu.addAction(act)
+        act = QAction("Set &Comment...", mnu)
+        act.triggered.connect(self.comment)
+        mnu.addAction(act)
+        mnu.addSeparator()
+        act = QAction("&Revert", mnu)
+        act.triggered.connect(self.revert_with_prompt)
+        mnu.addAction(act)
+        return mnu
+
+    def revert_with_prompt(self):
+        """
+        Revert this patch. Prompt for confirmation.
+        """
+        dlg = QMessageBox()
+        dlg.setWindowTitle("Revert patch")
+        dlg.setText("Are you sure you want to revert this patch?")
+        dlg.setIcon(QMessageBox.Question)
+        dlg.setStandardButtons(QMessageBox.Yes | QMessageBox.Cancel)
+        dlg.setDefaultButton(QMessageBox.Cancel)
+        if dlg.exec_() != QMessageBox.Yes:
+            return
+        self.revert()
+
+    def revert(self):
+        """
+        Revert this patch.
+        """
+        pm = self.view.workspace.instance.patches
+        pm.remove_patch(self.patch.addr)
+        pm.am_event()
+
+    def can_split(self) -> bool:
+        """
+        Determine if this patch can be split based on current cursor location.
+        """
+        cursor = self.view.inner_widget.hex.cursor
+        return self.patch.addr < cursor < (self.patch.addr + len(self.patch))
+
+    def split(self):
+        """
+        Split this patch at view's current cursor location.
+        """
+        cursor = self.view.inner_widget.hex.cursor
+        if self.can_split():
+            o = cursor - self.patch.addr
+            new_patch = Patch(cursor, self.patch.new_bytes[o:], self.patch.comment)
+            self.patch.new_bytes = self.patch.new_bytes[0:o]
+            pm = self.view.workspace.instance.patches
+            pm.add_patch_obj(new_patch)
+            pm.am_event()
+
+    def can_merge_with(self, other: 'PatchHighlightRegion') -> bool:
+        """
+        Determine if this patch can be merged with `other`. We only consider directly adjacent patches.
+        """
+        return other.patch.addr == (self.patch.addr + len(self.patch))
+
+    def merge_with(self, other: 'PatchHighlightRegion'):
+        """
+        Merge `other` into this patch.
+        """
+        if self.can_merge_with(other):
+            self.patch.new_bytes += other.patch.new_bytes
+            pm = self.view.workspace.instance.patches
+            pm.remove_patch(other.patch.addr)
+            pm.am_event()
+
+    def comment(self):
+        """
+        Set the comment for this patch.
+        """
+        dlg = InputPromptDialog('Set Patch Comment', 'Patch comment:', self.patch.comment, parent=self.view)
+        dlg.exec_()
+        if dlg.result:
+            self.patch.comment = dlg.result
+            pm = self.view.workspace.instance.patches
+            pm.am_event()
+
 
 class HexGraphicsObject(QGraphicsObject):
     """
     A graphics item providing a conventional hex-editor interface for a contiguous region of memory.
     """
 
-    cursor_changed = Signal(int)
-    context_menu_requested = Signal()
+    cursor_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -50,6 +158,7 @@ class HexGraphicsObject(QGraphicsObject):
         self.num_bytes: int = 0
         self.end_addr: HexAddress = 0  # Exclusive
         self.read_func: Optional[HexDataProvider] = None
+        self.write_func: Callable[[HexAddress, HexByteValue], bool] = None
         self.data: HexDataBuffer = b''
         self.addr_offset: int = 0
         self.addr_width: int = 0
@@ -67,6 +176,7 @@ class HexGraphicsObject(QGraphicsObject):
         self.row_padding: int = 0
         self.section_space: int = 0
         self.cursor: HexAddress = self.start_addr
+        self.cursor_nibble: Optional[int] = None
         self.selection_start: Optional[HexAddress] = None
         self.mouse_pressed: bool = False
         self.num_rows: int = 0
@@ -78,13 +188,13 @@ class HexGraphicsObject(QGraphicsObject):
 
         self.cursor_blink_timer: QTimer = QTimer(self)
         self.cursor_blink_timer.timeout.connect(self.toggle_cursor_blink)
-        self.restart_cursor_blink_timer()
         self.show_cursor: bool = True
         self._processing_cursor_update: bool = False
+        self.always_show_cursor: bool = False
 
         self._update_layout()
 
-    def focusInEvent(self, event:PySide2.QtGui.QFocusEvent):  # pylint: disable=unused-argument
+    def focusInEvent(self, event: PySide2.QtGui.QFocusEvent):  # pylint: disable=unused-argument
         """
         Item receives focus.
         """
@@ -92,14 +202,25 @@ class HexGraphicsObject(QGraphicsObject):
         self.restart_cursor_blink_timer()
         self.update()
 
-    def focusOutEvent(self, event:PySide2.QtGui.QFocusEvent):  # pylint: disable=unused-argument
+    def focusOutEvent(self, event: PySide2.QtGui.QFocusEvent):  # pylint: disable=unused-argument
         """
         Item lost focus.
         """
         self.cursor_blink_timer.stop()
-        self.show_cursor = False
+        self.show_cursor = self.always_show_cursor
+        self.cursor_blink_state = self.always_show_cursor
         self.update()
 
+    def set_always_show_cursor(self, always_show: bool):
+        """
+        Set policy of whether the cursor should always be shown (when focus is lost) or not.
+        """
+        self.always_show_cursor = always_show
+        if not self.cursor_blink_timer.isActive():
+            self.show_cursor = self.always_show_cursor
+            self.cursor_blink_state = self.always_show_cursor
+            self.update()
+
     def _set_data_common(self):
         """
         Common handler for set_data_*
@@ -119,12 +240,13 @@ class HexGraphicsObject(QGraphicsObject):
         self.read_func = None
         self._set_data_common()
 
-    def set_data_callback(self, read_func: HexDataProvider, start_addr: HexAddress, num_bytes: int):
+    def set_data_callback(self, write_func, read_func: HexDataProvider, start_addr: HexAddress, num_bytes: int):
         """
         Assign the buffer to be displayed using a callback function.
         """
         self.start_addr = start_addr
         self.num_bytes = num_bytes
+        self.write_func = write_func
         self.read_func = read_func
         self._set_data_common()
 
@@ -192,11 +314,15 @@ class HexGraphicsObject(QGraphicsObject):
         Get rect for address `addr` in whichever section is currently active.
         """
         if self.ascii_column_active:
-            pt = self.addr_to_point(addr, True)
-            return QRectF(pt, QSizeF(self.byte_width, self.row_height))
+            column_width = self.ascii_width
+            column_space = self.ascii_space
         else:
-            pt = self.addr_to_point(addr, False)
-            return QRectF(pt, QSizeF(self.ascii_width, self.row_height))
+            column_width = self.byte_width
+            column_space = self.byte_space
+
+        pt = self.addr_to_point(addr, self.ascii_column_active)
+        pt.setX(pt.x() - column_space / 2)
+        return QRectF(pt, QSizeF(column_width + column_space, self.row_height))
 
     def row_to_addr(self, row: int) -> int:
         """
@@ -239,7 +365,7 @@ class HexGraphicsObject(QGraphicsObject):
         """
         self.cursor_blink_timer.stop()
         self.cursor_blink_state = True
-        self.cursor_blink_timer.start(1000)
+        self.cursor_blink_timer.start(750)
 
     def update_active_highlight_regions(self):
         """
@@ -256,16 +382,23 @@ class HexGraphicsObject(QGraphicsObject):
             region_end = region.addr + region.size - 1
             region.active = not (region.addr > maxaddr or minaddr > region_end)
 
-    def get_highlight_region_under_cursor(self) -> Optional[HexHighlightRegion]:
+    def get_active_highlight_regions(self) -> Sequence[HexHighlightRegion]:
+        """
+        Get currently active highlight regions.
+        """
+        return [rgn for rgn in self.highlighted_regions if rgn.active]
+
+    def get_highlight_regions_under_cursor(self) -> Sequence[HexHighlightRegion]:
         """
         Return the region under the cursor, or None if there isn't a region under the cursor.
         """
+        regions = []
         for region in self.highlighted_regions:
             if region.addr <= self.cursor < (region.addr + region.size):
-                return region
-        return None
+                regions.append(region)
+        return regions
 
-    def set_cursor(self, addr: int, ascii_column: Optional[bool] = None):
+    def set_cursor(self, addr: int, ascii_column: Optional[bool] = None, nibble: Optional[int] = None):
         """
         Move cursor to address `addr`.
         """
@@ -276,11 +409,13 @@ class HexGraphicsObject(QGraphicsObject):
         self._processing_cursor_update = True
         if ascii_column is not None:
             self.ascii_column_active = ascii_column
-        self.restart_cursor_blink_timer()
+        if self.hasFocus():
+            self.restart_cursor_blink_timer()
         cursor_changed = self.cursor != addr
+        self.cursor = addr
+        self.cursor_nibble = nibble
+        self.cursor_changed.emit()
         if cursor_changed:
-            self.cursor = addr
-            self.cursor_changed.emit(self.cursor)
             self.update_active_highlight_regions()
             self.update()
         self._processing_cursor_update = False
@@ -302,17 +437,15 @@ class HexGraphicsObject(QGraphicsObject):
                 self.clear_selection()
             self.set_cursor(addr, ascii_column)
             event.accept()
-        if event.button() == Qt.RightButton:
-            self.context_menu_requested.emit()
-            event.accept()
 
-    def mouseDoubleClickEvent(self, event:QGraphicsSceneMouseEvent):
+    def mouseDoubleClickEvent(self, event: QGraphicsSceneMouseEvent):
         """
         Handle mouse double-click events (e.g. update selection)
         """
         if event.button() == Qt.LeftButton:
-            region = self.get_highlight_region_under_cursor()
-            if region is not None:
+            regions = sorted(self.get_highlight_regions_under_cursor(), key=lambda r: self.cursor - r.addr)
+            if len(regions) > 0:
+                region = regions[0]
                 self.set_cursor(region.addr + region.size - 1)
                 self.begin_selection()
                 self.set_cursor(region.addr)
@@ -336,35 +469,67 @@ class HexGraphicsObject(QGraphicsObject):
         """
         if event.button() == Qt.LeftButton:
             self.mouse_pressed = False
-            self.set_cursor(self.cursor)
 
-    def keyPressEvent(self, event:PySide2.QtGui.QKeyEvent):
+    def _set_byte_value(self, value: int):
+        """
+        Handle byte modification via user entry.
+        """
+        if not self.write_func(self.cursor, value):
+            return
+        self.set_cursor(self.cursor + 1)
+
+    def _set_nibble_value(self, value: int):
+        nibble = 1 if (self.cursor_nibble is None) else self.cursor_nibble
+        new_byte_value = self.read_func(self.cursor)
+        if not isinstance(new_byte_value, int):
+            new_byte_value = 0
+        else:
+            new_byte_value &= ~(0xF << (nibble * 4))
+        new_byte_value |= value << (nibble * 4)
+        if not self.write_func(self.cursor, new_byte_value):
+            return
+        next_nibble = 1 - nibble
+        self.set_cursor(self.cursor + next_nibble, nibble=next_nibble)
+
+    def keyPressEvent(self, event: PySide2.QtGui.QKeyEvent):
         """
         Handle key press events (e.g. moving cursor around).
         """
-        movement_keys = (Qt.Key_Up, Qt.Key_Down, Qt.Key_Right, Qt.Key_Left, Qt.Key_PageUp, Qt.Key_PageDown)
+        movement_keys = {
+            Qt.Key_Up: -16,
+            Qt.Key_Down: 16,
+            Qt.Key_Right: 1,
+            Qt.Key_Left: -1,
+            Qt.Key_PageUp: -8*16,
+            Qt.Key_PageDown: 8*16
+        }
         if event.key() in movement_keys:
-            if QApplication.keyboardModifiers() in (Qt.ShiftModifier,):
+            if int(QApplication.keyboardModifiers()) & Qt.ShiftModifier:
                 if self.selection_start is None:
                     self.begin_selection()
             else:
                 self.clear_selection()
-            new_cursor = self.cursor
-            if event.key() == Qt.Key_Up:
-                new_cursor -= 16
-            elif event.key() == Qt.Key_Down:
-                new_cursor += 16
-            elif event.key() == Qt.Key_Right:
-                new_cursor += 1
-            elif event.key() == Qt.Key_Left:
-                new_cursor -= 1
-            elif event.key() == Qt.Key_PageDown:
-                new_cursor += 8*16
-            elif event.key() == Qt.Key_PageUp:
-                new_cursor -= 8*16
+            new_cursor = self.cursor + movement_keys[event.key()]
             self.set_cursor(new_cursor)
             event.accept()
             return
+        elif event.key() == Qt.Key_Space and (int(QApplication.keyboardModifiers()) & Qt.ControlModifier):
+            self.set_cursor(self.cursor, ascii_column=(not self.ascii_column_active))
+            event.accept()
+            return
+        else:
+            t = event.text()
+            if len(t) == 1:
+                if self.ascii_column_active:
+                    if t.isascii() and t.isprintable():
+                        self._set_byte_value(ord(t))
+                        event.accept()
+                        return
+                else:
+                    if t in '0123456789abcdefABCDEF':
+                        self._set_nibble_value(int(t, 16))
+                        event.accept()
+                        return
         super().keyPressEvent(event)
 
     def _update_layout(self):
@@ -493,7 +658,7 @@ class HexGraphicsObject(QGraphicsObject):
         spath.closeSubpath()
         return spath
 
-    def get_value_for_addr(self, addr:int) -> Union[int, str]:
+    def get_value_for_addr(self, addr: int) -> Union[int, str]:
         """
         Get the value for given address `addr`.
         """
@@ -506,13 +671,11 @@ class HexGraphicsObject(QGraphicsObject):
         """
         Get active selection, returning (minaddr, maxaddr) inclusive.
         """
+        if self.selection_start is None:
+            return None
         if self.start_addr <= self.cursor < self.end_addr:
-            if self.selection_start is None:
-                minaddr = self.cursor
-                maxaddr = self.cursor
-            else:
-                minaddr = min(self.cursor, self.selection_start)
-                maxaddr = max(self.cursor, self.selection_start)
+            minaddr = min(self.cursor, self.selection_start)
+            maxaddr = max(self.cursor, self.selection_start)
             return (minaddr, maxaddr)
         return None
 
@@ -590,10 +753,10 @@ class HexGraphicsObject(QGraphicsObject):
 
             def set_pen_brush_for_active_selection(active):
                 if active:
-                    painter.setPen(QPen(Qt.red, pen_width))
+                    painter.setPen(QPen(Conf.hex_view_selection_color, pen_width))
                     painter.setBrush(QColor(255, 255, 255, 10))
                 else:
-                    painter.setPen(QPen(Qt.gray, pen_width))
+                    painter.setPen(QPen(Conf.hex_view_selection_alt_color, pen_width))
                     painter.setBrush(QColor(255, 255, 255, 10))
 
             set_pen_brush_for_active_selection(not self.ascii_column_active)
@@ -678,7 +841,11 @@ class HexGraphicsObject(QGraphicsObject):
             set_pen_brush_for_active_cursor(not self.ascii_column_active)
             tl = self.addr_to_point(self.cursor)
             tl.setY(tl.y() + self.row_height - cursor_height)
-            painter.drawRect(QRectF(tl, QSizeF(self.byte_width, cursor_height)))
+            cursor_width = self.byte_width
+            if self.cursor_nibble is not None:
+                cursor_width /= 2
+                tl.setX(tl.x() + cursor_width * (1 - self.cursor_nibble))
+            painter.drawRect(QRectF(tl, QSizeF(cursor_width, cursor_height)))
 
             # ASCII cursor
             set_pen_brush_for_active_cursor(self.ascii_column_active)
@@ -695,8 +862,7 @@ class HexGraphicsView(QGraphicsView):
     Container view for the HexGraphicsObject.
     """
 
-    cursor_changed = Signal(int)
-    context_menu_requested = Signal()
+    cursor_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -704,7 +870,6 @@ class HexGraphicsView(QGraphicsView):
         self._scene: QGraphicsScene = QGraphicsScene(parent=self)
         self.hex: HexGraphicsObject = HexGraphicsObject()
         self.hex.cursor_changed.connect(self.on_cursor_changed)
-        self.hex.context_menu_requested.connect(self.context_menu_requested.emit)
         self._scene.addItem(self.hex)
         self.setScene(self._scene)
 
@@ -713,20 +878,28 @@ class HexGraphicsView(QGraphicsView):
         self.setTransformationAnchor(QGraphicsView.NoAnchor)
         self.setAlignment(Qt.AlignTop | Qt.AlignLeft)
 
-    def set_region_callback(self, mem, addr, size):
+    def set_region_callback(self, write, mem, addr, size):
         s = self.scene().itemsBoundingRect()
         self.hex.setPos(s.bottomLeft())
-        self.hex.set_data_callback(mem, addr, size)
+        self.hex.set_data_callback(write, mem, addr, size)
         self.setSceneRect(self.scene().itemsBoundingRect())
 
-    def on_cursor_changed(self, addr: HexAddress):
+    def on_cursor_changed(self):
         """
         Handle cursor change events.
         """
-        self.cursor_changed.emit(addr)
-        target = self.sender().addr_to_rect(addr)
-        target.translate(self.sender().pos())
-        current = self.mapToScene(self.viewport().geometry()).boundingRect()
+        self.cursor_changed.emit()
+        self.move_viewport_to_cursor()
+
+    def move_viewport_to_cursor(self):
+        """
+        Ensure cursor is visible in viewport.
+        """
+        target = self.hex.addr_to_rect(self.hex.cursor)
+        target.translate(self.hex.pos())
+        vp_rect = self.viewport().geometry()
+        vp_rect.moveTo(0, 0)
+        current = self.mapToScene(vp_rect).boundingRect()
 
         if target.bottomRight().x() > current.bottomRight().x():
             dX = current.bottomRight().x() - target.bottomRight().x()  # Scroll right
@@ -745,7 +918,6 @@ class HexGraphicsView(QGraphicsView):
         if dX != 0 or dY != 0:
             self.translate(dX, dY)
 
-
     def adjust_viewport_scale(self, scale: Optional[float] = None):
         """
         Reset viewport scaling. If `scale` is None, viewport scaling is reset to default.
@@ -760,7 +932,7 @@ class HexGraphicsView(QGraphicsView):
         delta = new_pos - old_pos
         self.translate(delta.x(), delta.y())
 
-    def wheelEvent(self, event:QWheelEvent):
+    def wheelEvent(self, event: QWheelEvent):
         """
         Handle wheel events, specifically for changing font size when holding Ctrl key.
         """
@@ -778,7 +950,7 @@ class HexGraphicsView(QGraphicsView):
             self.setBackgroundBrush(Conf.palette_base)
             self.update()
 
-    def keyPressEvent(self, event:PySide2.QtGui.QKeyEvent):
+    def keyPressEvent(self, event: PySide2.QtGui.QKeyEvent):
         """
         Handle key events.
         """
@@ -798,55 +970,156 @@ class HexGraphicsView(QGraphicsView):
         super().keyPressEvent(event)
 
 
-class HexView(BaseView):
+class HexView(SynchronizedView):
     """
     View and edit memory/object code in classic hex editor format.
     """
 
+    _widgets_initialized: bool = False
+
     def __init__(self, workspace, default_docking_position, *args, **kwargs):
         super().__init__('hex', workspace, default_docking_position, *args, **kwargs)
         self.base_caption: str = 'Hex'
+        self.smart_highlighting_enabled: bool = True
+        self._clipboard = None
+        self._cfb_highlights: Sequence[HexHighlightRegion] = []
+        self._sync_view_highlights: Sequence[HexHighlightRegion] = []
+        self._patch_highlights: Sequence[HexHighlightRegion] = []
+
         self._init_widgets()
         self.workspace.instance.cfb.am_subscribe(self.reload_cfb)
+        self.workspace.instance.patches.am_subscribe(self._update_highlight_regions_from_patches)
 
-    def project_memory_read_func(self, addr:int) -> HexByteValue:
+        self.reload_cfb()
+        self._update_highlight_regions_from_patches()
+
+    def project_memory_read_func(self, addr: int) -> HexByteValue:
         """
         Callback to populate hex view with bytes.
         """
-        p = self.workspace.instance.project.am_obj
+        p = self.workspace.instance.project
+
+        patches = p.kb.patches.get_all_patches(addr, 1)
+        if len(patches) > 0:
+            patch = patches[0]
+            return patch.new_bytes[addr - patch.addr]
+
         try:
             return p.loader.memory[addr]
         except KeyError:
             return '?'
 
+    def auto_patch(self, addr: int, new_bytes: bytearray):
+        """
+        Automatically update or create patches to ensure new_bytes are patched at addr.
+        """
+        pm = self.workspace.instance.project.kb.patches
+        max_addr = addr + len(new_bytes) - 1
+
+        for p in pm.get_all_patches(addr, len(new_bytes)):
+            patch_max_addr = p.addr + len(p) - 1
+            if (p.addr <= addr) and (patch_max_addr >= max_addr):
+                # Existing patch contains new patch entirely. Update it.
+                p.new_bytes[(addr - p.addr):(max_addr - p.addr + 1)] = new_bytes
+                return
+            elif (p.addr >= addr) and (patch_max_addr <= max_addr):
+                # Patch will be entirely overwritten, remove it.
+                pm.remove_patch(p.addr)
+            elif (p.addr >= addr) and (patch_max_addr > max_addr):
+                # Lower portion of patch will be overwritten, shrink patch up.
+                pm.remove_patch(p.addr)
+                new_p_addr = max_addr + 1
+                p.new_bytes = p.new_bytes[(new_p_addr - p.addr):]
+                p.addr = new_p_addr
+                pm.add_patch_obj(p)
+            elif (p.addr < addr) and (patch_max_addr <= max_addr):
+                # Upper portion of patch will be overwritten, shrink patch down.
+                pm.remove_patch(p.addr)
+                p.new_bytes = p.new_bytes[0:(addr - p.addr)]
+                pm.add_patch_obj(p)
+            else:
+                assert False
+
+        # Check to see if we should extend an adjacent patch
+        if addr > 0:
+            p = pm.get_all_patches(addr - 1, 1)
+            if len(p) > 0:
+                p = p[0]
+                p.new_bytes += new_bytes
+                return
+
+        pm.add_patch_obj(Patch(addr, new_bytes))
+
+    def project_memory_write_bytearray(self, addr: int, value: bytearray) -> bool:
+        """
+        Callback to populate hex view with bytes.
+        """
+        self.auto_patch(addr, bytearray(value))
+        pm = self.workspace.instance.project.kb.patches
+        pm_notifier = self.workspace.instance.patches
+        if pm_notifier.am_none:
+            pm_notifier.am_obj = pm
+        pm_notifier.am_event()
+        return True
+
+    def project_memory_write_func(self, addr: int, value: int) -> bool:
+        """
+        Callback to populate hex view with bytes.
+        """
+        return self.project_memory_write_bytearray(addr, bytearray([value]))
+
     def reload_cfb(self):
         """
         Callback when project CFB changes.
         """
-        cfb = self.workspace.instance.cfb.am_obj
-        if cfb is None:
+        if self.workspace.instance.cfb.am_none:
             return
 
-        p = self.workspace.instance.project.am_obj
+        loader = self.workspace.instance.project.loader
         self.inner_widget.set_region_callback(
+            self.project_memory_write_func,
             self.project_memory_read_func,
-            p.loader.min_addr,
-            p.loader.max_addr - p.loader.min_addr + 1
+            loader.min_addr,
+            loader.max_addr - loader.min_addr + 1
         )
 
         self.inner_widget.cursor_changed.connect(self.on_cursor_changed)
 
+    def set_smart_highlighting_enabled(self, enable: bool):
+        """
+        Control whether smart highlighting is enabled or not.
+        """
+        self.smart_highlighting_enabled = enable
+        self._update_highlight_regions_under_cursor()
+
     def _init_widgets(self):
+        """
+        Initialize widgets for this view.
+        """
         window = QMainWindow()
         window.setWindowFlags(Qt.Widget)
 
         status_bar = QFrame()
         status_lyt = QHBoxLayout()
+        status_lyt.setContentsMargins(0, 0, 0, 0)
+
         self._status_lbl = QLabel()
         self._status_lbl.setText('Address: ')
+
         status_lyt.addWidget(self._status_lbl)
         status_lyt.addStretch(0)
-        status_lyt.setContentsMargins(0, 0, 0, 0)
+
+        option_btn = QPushButton()
+        option_btn.setText('Options')
+        option_mnu = QMenu(self)
+        smart_hl_act = QAction('Smart &highlighting', self)
+        smart_hl_act.setCheckable(True)
+        smart_hl_act.setChecked(self.smart_highlighting_enabled)
+        smart_hl_act.toggled.connect(self.set_smart_highlighting_enabled)
+        option_mnu.addAction(smart_hl_act)
+        option_btn.setMenu(option_mnu)
+        status_lyt.addWidget(option_btn)
+
         status_bar.setLayout(status_lyt)
 
         self.inner_widget = HexGraphicsView(parent=self)
@@ -855,20 +1128,174 @@ class HexView(BaseView):
         lyt.addWidget(status_bar)
         self.setLayout(lyt)
 
-    def on_cursor_changed(self, addr:int):
+        self._widgets_initialized = True
+
+    def revert_selected_patches(self):
         """
-        Handle updates to cursor
+        Revert any selected patches.
         """
-        s = 'Address: %08x' % addr
+        dlg = QMessageBox()
+        dlg.setWindowTitle("Revert patches")
+        dlg.setText("Are you sure you want to revert selected patches?")
+        dlg.setIcon(QMessageBox.Question)
+        dlg.setStandardButtons(QMessageBox.Yes | QMessageBox.Cancel)
+        dlg.setDefaultButton(QMessageBox.Cancel)
+        if dlg.exec_() != QMessageBox.Yes:
+            return
+
+        selected_regions = self.inner_widget.hex.get_active_highlight_regions()
+        for r in selected_regions:
+            if isinstance(r, PatchHighlightRegion):
+                r.revert()
+
+    def _can_merge_any_selected_patches(self):
+        """
+        Determine if any of the selected patches can be merged.
+        """
+        return self._merge_selected_patches(True)
+
+    def _merge_selected_patches(self, trial_only: bool = False) -> bool:
+        """
+        Merge selected directly-adjacent patches.
+        """
+        selected_patches = [r for r in self.inner_widget.hex.get_active_highlight_regions()
+                            if isinstance(r, PatchHighlightRegion)]
+        i = 0
+        did_patch = False
+        while i < (len(selected_patches) - 1):
+            patch = selected_patches[i]
+            for j, neighbor in enumerate(selected_patches[i+1:]):
+                if patch.can_merge_with(neighbor):
+                    if trial_only:
+                        return True
+                    patch.merge_with(neighbor)
+                    did_patch = True
+                else:
+                    i = i + j + 1
+                    break
+
+        return did_patch
+
+    def _get_num_selected_bytes(self) -> int:
+        """
+        Determine whether any bytes are selected.
+        """
+        sel = self.inner_widget.hex.get_selection()
+        if sel is None:
+            return 0
+        minaddr, maxaddr = sel
+        num_bytes_selected = maxaddr - minaddr + 1
+        return num_bytes_selected
+
+    def _copy_selected_bytes(self):
+        """
+        Copy selected bytes to view-only clipboard.
+        """
+        sel = self.inner_widget.hex.get_selection()
+        if sel is None:
+            self._clipboard = None
+            return
+
+        minaddr, maxaddr = sel
+        num_bytes_selected = maxaddr - minaddr + 1
+
+        self._clipboard = bytearray(num_bytes_selected)
+        for addr in range(minaddr, maxaddr + 1):
+            d = self.project_memory_read_func(addr)  # FIXME: Support multibyte read
+            if type(d) is int:
+                self._clipboard[addr-minaddr] = d
+
+    def _paste_copied_bytes_at_cursor(self):
+        """
+        Paste the view-only clipboard contents at cursor location.
+        """
+        if self._clipboard is not None:
+            self.project_memory_write_bytearray(self.inner_widget.hex.cursor, self._clipboard)
+
+    def contextMenuEvent(self, event: PySide2.QtGui.QContextMenuEvent):  # pylint: disable=unused-argument
+        """
+        Display view context menu.
+        """
+        mnu = QMenu(self)
+        add_sep = False
+
+        # FIXME: This should also go into an Edit menu accessible from the main window
+        num_selected_bytes = self._get_num_selected_bytes()
+        if num_selected_bytes > 0:
+            plural = "s" if num_selected_bytes != 1 else ""
+            act = QAction(f"Copy {num_selected_bytes:d} byte{plural}", mnu)
+            act.triggered.connect(self._copy_selected_bytes)
+            mnu.addAction(act)
+            add_sep = True
+        if self._clipboard is not None:
+            plural = "s" if len(self._clipboard) != 1 else ""
+            act = QAction(f"Paste {len(self._clipboard):d} byte{plural}", mnu)
+            act.triggered.connect(self._paste_copied_bytes_at_cursor)
+            mnu.addAction(act)
+            add_sep = True
+
+        if add_sep:
+            mnu.addSeparator()
+            add_sep = False
+
+        # Get context menu for specific item under cursor
+        for rgn in self.inner_widget.hex.get_highlight_regions_under_cursor():
+            rgn_mnu = rgn.gen_context_menu_actions()
+            if rgn_mnu is not None:
+                mnu.addMenu(rgn_mnu)
+                add_sep = True
+
+        if add_sep:
+            mnu.addSeparator()
+            add_sep = False
+
+        # Get context menu for groups of items
+        selected_regions = self.inner_widget.hex.get_active_highlight_regions()
+        if any(isinstance(r, PatchHighlightRegion) for r in selected_regions):
+            act = QAction('Merge selected patches', mnu)
+            act.triggered.connect(self._merge_selected_patches)
+            act.setEnabled(self._can_merge_any_selected_patches())
+            mnu.addAction(act)
+            act = QAction('Revert selected patches', mnu)
+            act.triggered.connect(self.revert_selected_patches)
+            mnu.addAction(act)
+            add_sep = True
+
+        if add_sep:
+            mnu.addSeparator()
+
+        mnu.addMenu(self.get_synchronize_with_submenu())
+        mnu.exec_(QCursor.pos())
+
+    def set_cursor(self, addr: int):
+        """
+        Move cursor to specific address and clear any active selection.
+        """
+        self.inner_widget.hex.clear_selection()
+        self.inner_widget.hex.set_cursor(addr)
+
+    def on_cursor_changed(self):
+        """
+        Handle updates to cursor.
+        """
+        self.update_status_text()
+        self._update_highlight_regions_under_cursor()
+        self.set_synchronized_cursor_address(self.inner_widget.hex.cursor)
+
+    def update_status_text(self):
+        """
+        Update status text with current cursor info.
+        """
+        s = 'Address: %08x' % self.inner_widget.hex.cursor
         sel = self.inner_widget.hex.get_selection()
         if sel is not None:
             minaddr, maxaddr = sel
             bytes_selected = maxaddr - minaddr + 1
-            if bytes_selected > 1:
-                s = 'Address: [%08x, %08x], %d bytes selected' % (minaddr, maxaddr, bytes_selected)
+            plural = "s" if bytes_selected != 1 else ""
+            s = f'Address: [{minaddr:08x}, {maxaddr:08x}], {bytes_selected} byte{plural} selected'
         self._status_lbl.setText(s)
 
-    def keyPressEvent(self, event:PySide2.QtGui.QKeyEvent):
+    def keyPressEvent(self, event: PySide2.QtGui.QKeyEvent):
         """
         Handle key events.
         """
@@ -879,9 +1306,101 @@ class HexView(BaseView):
         super().keyPressEvent(event)
 
     def popup_jumpto_dialog(self):
+        """
+        Display 'Jump To' dialog.
+        """
         JumpTo(self, parent=self).exec_()
 
-    def jump_to(self, addr:int) -> bool:
-        self.inner_widget.hex.clear_selection()
-        self.inner_widget.hex.set_cursor(addr)
+    def jump_to(self, addr: int) -> bool:
+        """
+        Jump to a specific address.
+        """
+        self.set_cursor(addr)
         return True
+
+    def on_synchronized_view_group_changed(self):
+        """
+        Handle view being added to or removed from the view synchronization group.
+        """
+        if self._widgets_initialized:
+            self.inner_widget.hex.set_always_show_cursor(len(self.sync_state.views) > 1)
+
+    def on_synchronized_highlight_regions_changed(self):
+        """
+        Handle synchronized highlight region change event.
+        """
+        self._update_highlight_regions_from_synchronized_views()
+
+    def _generate_highlight_regions_under_cursor(self) -> Sequence[HexHighlightRegion]:
+        """
+        Generate list of highlighted regions from CFB under cursor.
+        """
+        regions = []
+
+        try:
+            item = self.workspace.instance.cfb.floor_item(self.inner_widget.hex.cursor)
+        except KeyError:
+            item = None
+        if item is None:
+            return regions
+
+        addr, item = item
+        if self.inner_widget.hex.cursor >= (addr + item.size):
+            return regions
+
+        if isinstance(item, MemoryData):
+            color = Conf.hex_view_string_color if item.sort == 'string' else Conf.hex_view_data_color
+            regions.append(HexHighlightRegion(color, item.addr, item.size))
+        elif isinstance(item, Block):
+            for insn in item.disassembly.insns:
+                regions.append(HexHighlightRegion(Conf.hex_view_instruction_color, insn.address, insn.size))
+
+        return regions
+
+    def _update_highlight_regions_under_cursor(self):
+        """
+        Update cached list of highlight regions under cursor.
+        """
+        self._cfb_highlights = []
+        if self.smart_highlighting_enabled:
+            self._cfb_highlights.extend(self._generate_highlight_regions_under_cursor())
+        self._set_highlighted_regions()
+
+    def _generate_highlight_regions_from_synchronized_views(self) -> Sequence[HexHighlightRegion]:
+        """
+        Generate list of highlighted regions from any synchronized views.
+        """
+        regions = []
+        for v in self.sync_state.highlight_regions:
+            if v is not self:
+                for r in self.sync_state.highlight_regions[v]:
+                    regions.append(HexHighlightRegion(Qt.green, r.addr, r.size))
+        return regions
+
+    def _update_highlight_regions_from_synchronized_views(self):
+        """
+        Update cached list of highlight regions from synchronized views.
+        """
+        self._sync_view_highlights = self._generate_highlight_regions_from_synchronized_views()
+        self._set_highlighted_regions()
+
+    def _update_highlight_regions_from_patches(self):
+        """
+        Updates cached list of highlight regions from patches.
+        """
+        if self.workspace.instance.project.am_none:
+            self._patch_highlights = []
+        else:
+            self._patch_highlights = [PatchHighlightRegion(patch, self)
+                                      for patch in self.workspace.instance.project.kb.patches.values()]
+        self._set_highlighted_regions()
+
+    def _set_highlighted_regions(self):
+        """
+        Update highlighted regions, with data from CFB and synchronized views.
+        """
+        regions = []
+        regions.extend(self._cfb_highlights)
+        regions.extend(self._sync_view_highlights)
+        regions.extend(self._patch_highlights)
+        self.inner_widget.hex.set_highlight_regions(regions)
