@@ -58,13 +58,14 @@ class StringsView(BaseView):
         :return:
         """
 
-        if len(self.view_manager.views_by_category['disassembly']) == 1:
+        if len(self.workspace.view_manager.views_by_category['disassembly']) == 1:
             disasm_view = self.workspace.view_manager.first_view_in_category('disassembly')
         else:
             disasm_view = self.workspace.view_manager.current_view_in_category('disassembly')
-        disasm_view.jump_to(s.addr)
-        disasm_view.select_label(s.addr)
-        self.workspace.view_manager.raise_view(disasm_view)
+        if disasm_view is not None:
+            disasm_view.jump_to(s.addr)
+            disasm_view.select_label(s.addr)
+            self.workspace.view_manager.raise_view(disasm_view)
 
     def on_filter_change(self, **kwargs): #pylint: disable=unused-argument
         pattern = self._filter_string.text()
