@@ -1,6 +1,6 @@
 import os
 import string
-from typing import TYPE_CHECKING
+from typing import List, TYPE_CHECKING
 
 from angr.analyses.code_tagging import CodeTags
 
@@ -15,8 +15,13 @@ from ..toolbars import FunctionTableToolbar
 
 if TYPE_CHECKING:
     import PySide2
+    from angr.knowledge_plugins.functions import Function
+
 
 class QFunctionTableModel(QAbstractTableModel):
+    """
+    The table model for QFunctionTable.
+    """
 
     Headers = ['Name', 'Tags', 'Address', 'Binary', 'Size', 'Blocks']
     NAME_COL = 0
@@ -42,7 +47,7 @@ class QFunctionTableModel(QAbstractTableModel):
         return 0
 
     @property
-    def func_list(self):
+    def func_list(self) -> List['Function']:
         if self._func_list is not None:
             return self._func_list
         return self._raw_func_list
@@ -119,6 +124,10 @@ class QFunctionTableModel(QAbstractTableModel):
 
         elif role == Qt.BackgroundColorRole:
             color = self.workspace.plugins.color_func(func)
+            if color is None:
+                # default colors
+                if func.from_signature:
+                    color = self._config.function_table_signature_bg_color
             return color
 
         elif role == Qt.FontRole:
@@ -216,6 +225,9 @@ class QFunctionTableModel(QAbstractTableModel):
 
 
 class QFunctionTableView(QTableView):
+    """
+    The table view for QFunctionTable.
+    """
     def __init__(self, parent, workspace, selection_callback=None):
         super().__init__(parent)
         self.workspace = workspace
@@ -265,6 +277,11 @@ class QFunctionTableView(QTableView):
     def filter(self, keyword):
         self._model.filter(keyword)
 
+    def jump_to_result(self, index=0):
+        if len(self._model.func_list) > index:
+            self._selected_func.am_obj = self._model.func_list[index]
+            self._selected_func.am_event(func=self._selected_func.am_obj)
+
     def load_functions(self):
         if not self.show_alignment_functions:
             self._model.func_list = [ v for v in self._functions.values() if not v.alignment ]
@@ -288,11 +305,15 @@ class QFunctionTableView(QTableView):
         return True
 
     def contextMenuEvent(self, event:'PySide2.QtGui.QContextMenuEvent') -> None: # pylint:disable=unused-argument
-        row = self.currentIndex().row()
-        self._context_menu.set(self._model.func_list[row]).qmenu().popup(QCursor.pos())
+        rows = self.selectionModel().selectedRows()
+        funcs = [self.workspace.instance.kb.functions[r.data()] for r in rows]
+        self._context_menu.set(funcs).qmenu().popup(QCursor.pos())
 
 
 class QFunctionTableFilterBox(QLineEdit):
+    """
+    The filter box for QFunctionTable.
+    """
     def __init__(self, parent):
         super().__init__()
 
@@ -315,6 +336,9 @@ class QFunctionTableFilterBox(QLineEdit):
 
 
 class QFunctionTable(QWidget):
+    """
+    Implements a table for listing function details.
+    """
 
     def __init__(self, parent, workspace, selection_callback=None):
         super().__init__(parent)
@@ -427,5 +451,6 @@ class QFunctionTable(QWidget):
         self.filter_functions(text)
 
     def _on_filter_box_return_pressed(self):
+        self._table_view.jump_to_result()
         # Hide the filter box
         self.hide_filter_box()
